#!/usr/bin/perl

#use Math::BigFloat;

$tracr = shift @ARGV;
$in = shift @ARGV;
$temp = int(rand(1000));

open FA, $in;

while(<FA>){
        if($_ =~ /^>(\w+);?.*\n/){
                $name = $1;
                next;
        }
        chomp;
        $fasta{$name} .= $_;
}
close FA;

foreach $key (keys(%fasta)){
    #input format:
    #2  %number of sequences
    #seq1
    #seq2
    #2  %number of strands allowed in complexes
    
    open OUT, ">$temp.in";
    print OUT "2\n$tracr\n$fasta{$key}\n2";
    close OUT;

    #concentration format:
    #con1
    #con2

    open OUT, ">$temp.con";
    print OUT "1e-6\n1e-6";
    close OUT;
    
    print STDERR "Running: complexes -mfe $temp\n";
    system("complexes -mfe $temp >&2") == 0 or die "complexes failed: $!\n";
    print STDERR "Running: concentrations -sort 0 $temp\n";
    system("concentrations -sort 0 $temp >&2") == 0 or die "concentrations failed: $!\n";

    #need: concentration of each species ($temp.eq)
    #test.eq format: complex\tordering\tn mol 1 (tracr)\tn mol2 (target)\tmfe\tconc
    open IN, "$temp.eq";
    while(<IN>){
        if($_ =~ /^%/){
            next;
        }
        chomp;
        @dat = split "\t";
        if($dat[2] == 1 && $dat[3] == 0){
            $dhash{$key}{"tracr_conc"} = $dat[5];
        } elsif ($dat[2] == 0 && $dat[3] == 1){
            $dhash{$key}{"target_conc"} = $dat[5];
        } elsif ($dat[2] == 2 && $dat[3] == 0){
            $dhash{$key}{"tracr_duplex_conc"} = $dat[5];
        } elsif ($dat[2] == 1 && $dat[3] == 1){
            $dhash{$key}{"tracr_target_duplex_conc"} = $dat[5];
            $duplex_comp = $dat[0];
        } elsif ($dat[2] == 0 && $dat[3] == 2){
            $dhash{$key}{"target_duplex_conc"} = $dat[5];
        }
    }

    close IN;

    foreach $key2 ("tracr_conc", "target_conc", "tracr_duplex_conc", "tracr_target_duplex_conc", "target_duplex_conc"){
        die "Key $key2 for $key does not exist!\n" if(!exists($dhash{$key}{$key2}));
    }
    
    #need: interacting bases ($temp.ocx-mfe)
    #block format:
    #% compositionN-orderingI
    #number of bases in complex
    #MFE
    #dot-bracket -- + separates two molecules (tracr+target)
    #base pair

    open IN, "$temp.ocx-mfe";
    while(<IN>){
        next if($_ !~ /^% composition$duplex_comp-ordering1/);

        readline(IN); #thow away bp
        $mfe = readline(IN);
        $db = readline(IN);
        $db =~ /\+/ or die("+ not found in $name dot-bracket notation\n");
        $split_pos = $-[0];
        
        while(<IN>){
            last if($_ =~ /^%/);
            @splat = split "\t";
            if(($splat[0] < $split_pos) & ($splat[1] > $split_pos)){ #exclude intramolecular base-pairs
                $dhash{$key}{'bps'}{$splat[0]} = $splat[1] - $split_pos;
            }
        }
        last;
    }
    system("rm $temp.*") == 0 or die "Couldn't remove temp files: $!\n";
}


print "target_seq\ttracr_conc\ttarget_conc\ttracr_duplex_conc\ttarget_duplex_conc\ttracr_target_duplex_conc\teq_binding\ttracer_bps\ttarget_bps\n";
foreach $key (keys(%dhash)){
    print $key,"\t",$dhash{$key}{"tracr_conc"}, "\t", $dhash{$key}{"target_conc"}, "\t",$dhash{$key}{"tracr_duplex_conc"}, "\t", $dhash{$key}{"target_duplex_conc"}, "\t", $dhash{$key}{"tracr_target_duplex_conc"}, "\t";
    
    
    $duplex = sprintf("%.20f", $dhash{$key}{"tracr_target_duplex_conc"});
    $tracr = sprintf("%.20f", $dhash{$key}{"tracr_conc"});
    $target = sprintf("%.20f", $dhash{$key}{"target_conc"});

    $eq = $duplex / ($tracr * $target);
    print sprintf("%1.6e", $eq),"\t";
    $tracr_bps = "";
    $target_bps = "";
    foreach $bp (sort { $a <=> $b} (keys %{$dhash{$key}{'bps'}})){
        $tracr_bps .= $bp . ",";
        $target_bps .= $dhash{$key}{'bps'}{$bp} . ",";
    }
    chop $tracr_bps;
    chop $target_bps;
    print $tracr_bps, "\t", $target_bps,"\n";
} 
